function varargout =DA_narrow2band_spectrum(xSpectr,nFreqs,OctOrThird,varargin)
% [bX,bFreqs]=DA_narrow2band_spectrum(xSpectr,nFreqs,OctOrThird,varargin)
%
% Data Acoustics - Converter routine
%
% Generates Octave or Third octave, or finer band spectra from narrow band spectra.
% <OctOrThird> must be one of the followings: 
%    '1/1', '1/2', '1/3', '1/6', '1/12', '1/24'
%
%
% xSpectr:      Matrix, with spectras of the channels in its COLUMNs
% nFreqs:       Column, with the frequency values of the spectral lines
% OctOrThird:   must be one of the followings: '1/1', '1/2', '1/3', '1/6', '1/12', '1/24'
%
% varargin:
%   '-frf':     specify this, if your input is a transfer function, rahter then an autospectrum
%               (In this case each band's value must be divided by the number of spectral lines
%               contributing to the given band's value. This option lets perform the correction.)
%   '-lpb' <n>: Spectral band is valid if contains at least <n> spectral lines. Default = 5.
%   '-flimits' [fmin, fmax]: band spectra is asked for this range only
%
% bX:           Matrix, contains the Octave, Third Octave, etc. band spectras generated from xSpectr.
%               (linear values, not dBs)
% bFreqs:       Frequency values for each line of the spectras.
%
% Notes:
% (1):  highest band center is computed to be below max(nFreqs), 
%       spectral components beyound this band's upper frequency limit are ommited
% (2):  spectral components contributing to invalid bands are also ommited
%       (those not reaching the minimum required numbers of bins in a band)
% (3):  Definition: X(band) = sqrt( szum( X(k) * conj(X(k)) ) )
% 
% A.B. Nagy, F. Marki - 2003-2018.

% v2.1
% 'conj' beszrsa az 51-es sorba

% v3.1
% sv-frekvencik generlsa DA-GenOctBandFreqs-be ttve
% sszegz algoritmus javtsa

prg_ver='3.1';
isFRF = 0;
energyCORR = 1;
min_lines_per_band = 5;

ERR = 'DA_narrow2band_spectrum error';

if ischar(xSpectr)
  if  strcmp(xSpectr,'-rms_spectr')
      if strcmp(nFreqs(end-10:end),'_rms_spectr')
          matfile_spectr=[nFreqs(1:end-11) '_3rd_spectr.mat'];
      else
          matfile_spectr=[nFreqs '_3rd_spectr.mat'];
      end
      matfile = [nFreqs '.mat'];
  else
      matfile = [xSpectr '_temp.mat'];
  end
  clear xSpectr;
  
  try
    if ~exist(matfile,'file')                                               % check the file
        error(['Frequency data file (' matfile ') not be found!']);
    end
 %   hWaitbar=waitbar(0,'Reading frequency data file...');
%    set(hWaitbar,'Name',['DA_narrow2band_spectrum - ' prg_ver]);
    load('-mat', matfile);                                                  % load database
    if ~exist('X','var')
%        close(hWaitbar);
        matfile
        error('Invalid file specified. Frequency data doesn''t not exist!');
    end
    xSpectr=X;
  catch MException
    errordlg(MException.message);        
  end
else
    if isstruct(nFreqs)
        info=nFreqs;
        nFreqs=info.nFreqs;
    end
 %   hWaitbar=waitbar(0,'Computing narrow band spectra...');
%    set(hWaitbar,'Name',['DA_wav2spect - ' prg_ver]);
end    

isDataToProcess=1;
for n=1:nargin-3
    if isDataToProcess
        switch lower(varargin{n}) 
            case '-frf'
                isFRF=1;
                disp('FRF correction will be applied.');
            case '-energy_corr'
                if nargin-3 < n+1
                    disp('ERROR: Energy correction value doesn''t exist');
                    return;
                end
                energyCORR=varargin{n+1};
                if ~isnumeric(energyCORR)
                    disp('ERROR: Energy correction should be a numeric value');
                    return;
                end
                isDataToProcess=0;
            case '-lpb'
                if nargin-3 < n+1
                    disp('ERROR: Lines Per Band data doesn''t exist');
                    return;
                    
                end
                min_lines_per_band=varargin{n+1};
                if ~isnumeric(min_lines_per_band) || round(min_lines_per_band)~=min_lines_per_band || min_lines_per_band<=0
                    disp('ERROR: Lines Per Band data should be positive integer');
                    return;
                end
                isDataToProcess=0;
            case '-flimits'
                if nargin-3 < n+1
                    disp('ERROR: Frequency limits values are missing');
                    return;
                end
                if ~isnumeric(varargin{n+1})
                    disp('ERROR: Frequency limits should be a numeric value');
                    return;
                end
                if sum( size(varargin{n+1})==[1,2] ) ~= 2
                    disp('ERROR: Frequency limits value must be a row vectors with two elements');
                    return;
                end
                fLimits = varargin{n+1};
                isDataToProcess=0;
            otherwise
                disp(['WARNING: Unrecognized option ''' varargin{n} '''']);
        end
    else
        isDataToProcess=1;
    end
end

if ~exist('isFRF','var'),   isFRF=0;  end

if min(size(nFreqs))~=1
    disp('ERROR: Frequency data must be a row or coloumn VECTOR!'); 
    return;
end
if size(xSpectr,1)~=length(nFreqs)
    xSpectr=xSpectr.';
    disp('WARNING: Spectrum data should be a coloumn vector or coloumns of a matrix'); 
end
if size(xSpectr,1)~=length(nFreqs)
    disp('ERROR: Length of frequency vector must be equal to the size of spectrum/-a vector(s)'); 
    return;
end

if nFreqs(1)==0, xSpectr(1,:)=[]; nFreqs(1)=[]; end

% sv-frekvencik lekrse - a DC komponens mr nincs benne, ezrt j
% fmin-nek nFreqs(1)
if ~exist('fLimits','var')
    fLimits = [nFreqs(1), nFreqs(end)];
end
[bFreqs, bFreqsExact, fLow, fHigh] = DA_GenOctBandFreqs(OctOrThird, fLimits(1), fLimits(2));
% [fLow(1); fHigh']
% bFreqsExact'

% ---------------------- tercs�v k�z�pfrekvenci�k elk�sz�ltek ---------------------------

[nFreqLines,nCh]=size(xSpectr);
bX2=zeros(length(bFreqs),nCh);
binsInBand=zeros(length(bFreqs),1);

band_no=1; fStart=1; %start_fline=0;

% frequency bins below the first band are omitted
while nFreqs(fStart) < fLow(1)
    fStart=fStart+1;
end

for f=fStart:nFreqLines
    % omit frequencies beyound the last band
    if nFreqs(f) > fHigh(end)
        break
    end
    % increase band number until freq. bin falls into the correct band
    while nFreqs(f) > fHigh(band_no)
            band_no=band_no+1;
%             start_fline(band_no)=f;    
    end
    % A k�vetkez� sorban kell a conj, hogy komplex spektrumokat is lehessen haszn�lni    
    % Negyzetosszeg generalasa egy savra, az �sszes spektrumra k�l�n-k�l�n
    bX2(band_no,:)=bX2(band_no,:) + xSpectr(f,:).*conj(xSpectr(f,:));

    % A kovetkezoben szamolom, hogy egy savba hany spektrumvonal esett
    binsInBand(band_no)=binsInBand(band_no)+1;
end

% binsInBand

invalid_bands=find(binsInBand<min_lines_per_band);    % Megkeress�k az �rv�nyes frekvencias�vokat
bX2(invalid_bands,:) = [];
binsInBand(invalid_bands) = [];
bFreqs(invalid_bands) = [];

% Ha FRF-r�l van sz�, akkor le kell osztani a s�vba es� spektrumvonalak sz�m�val
if isFRF
    for k=1:length(bFreqs)
        bX2(k,:)=bX2(k,:)/binsInBand(k);    
    end
end
% Es vegul johet a gyokvonas
if energyCORR~=1
    disp(['Energy correction will be applied: ' num2str(energyCORR) ]);
    bX=sqrt(bX2*energyCORR);
else
    bX=sqrt(bX2);
end

if exist('info','var')
    info.History(end+1) = { ['DA_narrow2band_spectrum. Bandwidth: ' OctOrThird]  };
end
if nargout>1
    varargout{1}=bX;
    if exist('info','var')
        info.bFreqs=bFreqs;
        varargout{2}=bFreqs;
        varargout{3}=info;
    else
        varargout{2}=bFreqs;
%         varargout{3}=start_fline;
    end
    
else
%    waitbar(1, hWaitbar, 'Saving results...');
    save(matfile_spectr,'info','bFreqs','bX');
    varargout{1}=info;
end
%close(hWaitbar);
